import { httpApp } from "@/plugins/axiosApp";
import { HistoryQueryParams, Question } from "@/types";

/** 清理复核任务 */
export async function clearInspectedTask(
  subjectCode?: string,
  groupNumber?: string
) {
  const form = new FormData();
  subjectCode && form.append("subjectCode", subjectCode);
  groupNumber && form.append("groupNumber", groupNumber);
  return httpApp.post("/admin/exam/library/clear", form);
}

/** 查看单个学生的复核任务 */
// export async function getSingleInspectedTask(studentId: string) {
//   // return httpApp.post("/admin/exam/inspected/getTask?studentId=" + studentId);
//   const form = new FormData();
//   studentId && form.append("studentId", studentId);
//   return httpApp.post("/admin/exam/inspected/getTask", form);
// }

/** 批量复核历史 */
export async function getInspectedHistory({
  subjectCode,
  order = "markerTime",
  sort = "DESC",
  pageNumber = 1,
  pageSize = 20,
}: HistoryQueryParams) {
  if (!subjectCode) return;
  const form = new FormData();
  form.append("subjectCode", subjectCode);
  form.append("order", order);
  form.append("sort", sort);
  form.append("pageNumber", pageNumber + "");
  form.append("pageSize", pageSize + "");
  return httpApp.post("/admin/exam/library/getHistory", form);
}

/** 批量复核得到单个学生的复核任务 */
export async function getOneOfInspectedTask(
  subjectCode?: string,
  groupNumber?: string
) {
  const form = new FormData();
  subjectCode && form.append("subjectCode", subjectCode);
  groupNumber && form.append("groupNumber", groupNumber);
  return httpApp.post("/admin/exam/library/getTask", form);
}

/** 批量复核得到任务总数 */
export async function getInspectedTaskStatus(
  subjectCode: string,
  groupNumber: string
) {
  const form = new FormData();
  form.append("subjectCode", subjectCode);
  form.append("groupNumber", groupNumber);
  return httpApp.post("/admin/exam/library/getStatus", form);
}

/** 批量复核设置 */
export async function getInspectedSetting() {
  return httpApp.post("/admin/exam/inspected/getSetting");
}

/** 保存复核任务 */
export async function saveInspectedTask(libraryId: string) {
  const form = new FormData();
  form.append("libraryId", libraryId);
  return httpApp.post("/admin/exam/library/inspected/save", form);
}

/** 复核任务打回问题 */
export async function rejectInspectedTask(
  libraryId: string,
  questionList: Array<Question>
) {
  return httpApp.post("/admin/exam/library/rejected", {
    libraryId,
    questionList,
  });
}
