export interface MarkStore {
  setting: Setting;
  groups: Array<Group>;
  status: {
    personCount: number; //个人评卷数量
    markedCount: number; //总评卷数量
    totalCount: number; //总数量
    problemCount: number; //问题卷数量
    arbitrateCount: number; //待仲裁卷数量

    personCountTweenedNumber: number; // 已评数量（动画用）
    todoMarkCountTweenedNumber: number; // 待评数量（动画用）
  };
  tasks: Array<Task>; // 保持数量为3
  currentTask?: Task; // 用来切换task，还有回看
  currentQuestion?: Question;
  currentScore?: number;
  currentSpecialTag?: string;
  historyOpen: boolean; // 是否打开回评侧边栏
  historyTasks: Array<Task>;
  removeScoreTracks: Array<Track>;
  focusTracks: Array<Track>; // 暂时无用
  message: string | null;
  maxModalZIndex: number;
  minimapScrollToX?: number; // 宽度的百分比
  minimapScrollToY?: number; // 高度的百分比
  allPaperModal: boolean; // 是否显示全卷
  sheetViewModal: boolean; // 是否显示原卷
  globalMask: boolean; // 是否全局遮盖
}

export interface Setting {
  examType: "SCAN_IMAGE" | "MULTI_MEDIA"; // 扫描图片或者多媒体，多媒体只允许 common mode
  mode: ModeEnum; //模式 TRACK/COMMON
  forceMode: boolean; //强制模式切换
  sheetView: boolean; //是否显示原图功能
  sheetConfig: Array<SingleSheetConfig>; //原图遮盖规则
  enableAllZero: boolean; // 是否开启全零分
  fileServer: string; //图片服务地址
  userName: string; //评卷员姓名
  subject: {
    //科目信息（试卷和答案功能）
    name: string;
    code: string;
    answerUrl: string;
    paperUrl: string;
    questions: Array<RichTextQuestion>;
  };
  forceSpecialTag: boolean; //强制标记是否开启 forceTag
  uiSetting: UISetting;
  statusValue: "TRIAL" | "FORMAL" | "FINISH"; //只显示试评名称  TRIAL("试评"), FORMAL("正评"), FINISH("结束")
  problemTypes: Array<{ id: number; name: string }> | []; //问题卷类型
  groupNumber: number; //评卷分组
  groupTitle: string; //评卷分组名称
  topCount: number; //限制提示数量
  splitConfig: Array<number>; //使用裁切整图时的裁切配置 [0,1]|[0,0.3,0.25,0.55],
  prefetchCount: number; //预加载任务数量
  startTime: number; //评卷开始时间
  endTime: number; //评卷结束时间
  selective: boolean; // 是否是未选择类型
}

export enum ModeEnum {
  TRACK = "TRACK",
  COMMON = "COMMON",
}

interface SingleSheetConfig {
  i: number; // index of sheets
  w: number; // width
  h: number; // height
  x: number; // x 左上角为 (0, 0)
  y: number; // y 左上角为 (0, 0)
}

interface Group {
  markerId: number;
  number: number;
  title: string;
  markedCount: number; //总评卷数量
  totalCount: number; //总数量
}

export interface Task {
  libraryId: number;
  studentId: number;
  secretNumber: string;
  studentName: string; //后端处理是否显示
  studentCode: string;
  examNumber: string;
  // subject: { name: string; code: string };
  sliceUrls: Array<string>; //裁切图url
  sliceConfig: Array<PictureSlice>; //最高显示优先级
  jsonUrl: string; // sliceUrls为空，则是多媒体阅卷，显示JSON
  questionList: Array<Question>;
  specialTagList: Array<SpecialTag>;

  sheetUrls: Array<string>; //原图url
  objectiveScore: number; //客观分 复核也用到

  markerScore: number; // 评卷总分
  markerTime: number; //评卷时间
  inspectTime?: number; // 复核有用
  self: boolean; // 是否自评，暂时用不着
  previous: boolean; // 是否回评
  rejected: boolean; // 是否是打回
  message: string | null;

  // 评卷结果，在task第一次被访问时自动添加，watch currentTask
  markResult: MarkResult;
  __markStartTime: number; // 前端自用，用于标记阅卷开始时间和计算spent
}

export interface Question {
  groupNumber: number; // 分组序号
  mainNumber: number; // 大题号
  subNumber: string; // 小题号
  intervalScore: number; // 分数间隔
  defaultScore: number;
  minScore: number;
  maxScore: number;
  title: string; // 题目名称
  trackList: Array<Track>; // 轨迹列表
  score: number | null; //得分；null的值时是为打回时可以被评卷修改的；null也是从未评分过的情况，要通过rejected来判断
  __index: number; // question 在 task 里面的 index ，用来对应 scoreList 的 score
}

export interface Track {
  mainNumber: number; // 大题号
  subNumber: string; // 小题号，当前api中只有number // 特殊标记中没有
  number: number; // 前端使用，暂时用不着，赋0
  offsetIndex: number; // 第几张图
  offsetX: number; // 左上角为原点
  offsetY: number;
  positionX: number; // 相对slice的位置比例
  positionY: number;
  score: number;
}

export interface SpecialTag {
  offsetIndex: number; // 第几张图
  offsetX: number; // 左上角为原点
  offsetY: number;
  positionX: number; // 相对slice的位置比例
  positionY: number;
  tagName: string; // 特殊标记的字符串，勾叉
}

export interface PictureSlice {
  i: number; // 从1开始
  w: number;
  h: number;
  x: number;
  y: number;
}

export interface UISetting {
  "score.board.collapse": boolean;
  "answer.paper.scale": number; // 0.2 gap
  "normal.mode": "keyboard" | "mouse";
  "paper.modal": boolean;
  "answer.modal": boolean;
  "minimap.modal": boolean;
  "specialTag.modal": boolean;
  "score.fontSize.scale": number; // 0.1 gap
}

export interface MarkResult {
  libraryId: number;
  studentId: number;
  statusValue: string;
  spent: number; // 毫秒单位

  // 轨迹 or 键盘
  markerScore: number | null;
  trackList: Array<Track>;
  scoreList: Array<number | null>;
  specialTagList: Array<SpecialTag>; // 轨迹和键盘都需要

  // 问题卷
  problem: boolean;
  problemTypeId: number;
  unselective: boolean; // 当前task是否为学生未选做
}

// 前端自用，用来渲染裁切图
export interface SliceImage {
  url: string; // 当前是 ObjectURL , 因为 DataURL 性能太差
  indexInSliceUrls: number;
  trackList: Array<Track>;
  tagList: Array<SpecialTag>;
  // originalImageWidth: number; // 为了兼容原图还原轨迹而添加的属性，当前CommonMarkBody用不到
  // originalImageHeight: number; // 为了兼容原图还原轨迹而添加的属性，当前CommonMarkBody用不到
  sliceImageWidth: number;
  sliceImageHeight: number;
  dx: number; // 裁切图在原图中的左上角的x偏移量
  dy: number; // 裁切图在原图中的左上角的y偏移量
  accumTopHeight: number; // 在多个图片从高至低排列中中累积的高度
  effectiveWidth: number;
}

export type MarkHistoryOrderBy =
  | "markerTime"
  | "inspectTime"
  | "markerScore"
  | "seceretNumber"
  | undefined;
export type MarkHistorySortField = "ASC" | "DESC" | undefined;

export interface HistoryQueryParams {
  pageNumber?: number; // 从1开始
  pageSize?: number;
  order?: MarkHistoryOrderBy;
  sort?: MarkHistorySortField;
  secretNumber?: string | null;
  subjectCode?: string;
  groupNumber?: string;
  markerId?: string;
  markerScore?: string;
}

export interface getHistory {
  (historyQuery: HistoryQueryParams): any;
}

export interface RichTextQuestion {
  unionOrder: string; // 题目的综合题号 1-2-4
  body: RichTextJSON;
  parentBody: RichTextJSON | null;
  answer: Array<RichTextJSON> | null;
  objective: boolean | null;
  options: Array<{ number: number; body: RichTextJSON }>;
}
export interface RichTextJSON {
  sections: RichTextSectionJSON[];
}
export interface RichTextSectionJSON {
  blocks: RichTextBlockJSON[];
}
export interface RichTextBlockJSON {
  type: "text" | "image" | "audio" | "cloze";
  value: string;
  param: {
    underline: boolean;
    bold: boolean;
    italic: boolean;
    width: string;
    height: string;
  } | null;
}
